@@ -1,18 +1,19 @@
-from alembic.script import ScriptDirectory
-from alembic.environment import EnvironmentContext
-from alembic import util, ddl, autogenerate as autogen
 import os
 
+from .script import ScriptDirectory
+from .environment import EnvironmentContext
+from . import util, autogenerate as autogen
+
 def list_templates(config):
     """List available templates"""
 
     config.print_stdout("Available templates:\n")
     for tempname in os.listdir(config.get_template_directory()):
-        readme = os.path.join(
+        with open(os.path.join(
                         config.get_template_directory(),
                         tempname,
-                        'README')
-        synopsis = open(readme).next()
+                        'README')) as readme:
+            synopsis = readme.next()
         config.print_stdout("%s - %s", tempname, synopsis)
 
     config.print_stdout("\nTemplates are used via the 'init' command, e.g.:")
@@ -76,7 +77,6 @@ def revision(config, message=None, autogenerate=False, sql=False):
 
     if autogenerate:
         environment = True
-        util.requires_07("autogenerate")
         def retrieve_migrations(rev, context):
             if script.get_revision(rev) is not script.get_revision("head"):
                 raise util.CommandError("Target database is not up to date.")
@@ -149,14 +149,49 @@ def downgrade(config, revision, sql=False, tag=None):
     ):
         script.run_env()
 
-def history(config):
+def history(config, rev_range=None):
     """List changeset scripts in chronological order."""
 
     script = ScriptDirectory.from_config(config)
-    for sc in script.walk_revisions():
-        if sc.is_head:
-            config.print_stdout("")
-        config.print_stdout(sc)
+    if rev_range is not None:
+        if ":" not in rev_range:
+            raise util.CommandError(
+                    "History range requires [start]:[end], "
+                    "[start]:, or :[end]")
+        base, head = rev_range.strip().split(":")
+    else:
+        base = head = None
+
+    def _display_history(config, script, base, head):
+        for sc in script.walk_revisions(
+                                base=base or "base",
+                                head=head or "head"):
+            if sc.is_head:
+                config.print_stdout("")
+            config.print_stdout(sc.log_entry)
+
+    def _display_history_w_current(config, script, base=None, head=None):
+        def _display_current_history(rev, context):
+            if head is None:
+                _display_history(config, script, base, rev)
+            elif base is None:
+                _display_history(config, script, rev, head)
+            return []
+
+        with EnvironmentContext(
+            config,
+            script,
+            fn=_display_current_history
+        ):
+            script.run_env()
+
+    if base == "current":
+        _display_history_w_current(config, script, head=head)
+    elif head == "current":
+        _display_history_w_current(config, script, base=base)
+    else:
+        _display_history(config, script, base, head)
+
 
 def branches(config):
     """Show current un-spliced branch points"""
@@ -170,15 +205,23 @@ def branches(config):
                     script.get_revision(rev)
                 )
 
-def current(config):
+def current(config, head_only=False):
     """Display the current revision for each database."""
 
     script = ScriptDirectory.from_config(config)
     def display_version(rev, context):
-        config.print_stdout("Current revision for %s: %s",
-                            util.obfuscate_url_pw(
-                                context.connection.engine.url),
-                            script.get_revision(rev))
+        rev = script.get_revision(rev)
+
+        if head_only:
+            config.print_stdout("%s%s" % (
+                rev.revision if rev else None,
+                " (head)" if rev and rev.is_head else ""))
+
+        else:
+            config.print_stdout("Current revision for %s: %s",
+                                util.obfuscate_url_pw(
+                                    context.connection.engine.url),
+                                rev)
         return []
 
     with EnvironmentContext(
@@ -220,5 +263,3 @@ def splice(config, parent, child):
 
     """
     raise NotImplementedError()
-
-
