@@ -1,10 +1,11 @@
-from alembic import command, util, package_dir
 from argparse import ArgumentParser
-import ConfigParser
+from .compat import SafeConfigParser
 import inspect
 import os
 import sys
 
+from . import command, util, package_dir, compat
+
 class Config(object):
     """Represent an Alembic configuration.
 
@@ -50,7 +51,7 @@ class Config(object):
 
     """
     def __init__(self, file_=None, ini_section='alembic', output_buffer=None,
-                        stdout=sys.stdout):
+                        stdout=sys.stdout, cmd_opts=None):
         """Construct a new :class:`.Config`
 
         """
@@ -58,6 +59,21 @@ class Config(object):
         self.config_ini_section = ini_section
         self.output_buffer = output_buffer
         self.stdout = stdout
+        self.cmd_opts = cmd_opts
+
+    cmd_opts = None
+    """The command-line options passed to the ``alembic`` script.
+
+    Within an ``env.py`` script this can be accessed via the
+    :attr:`.EnvironmentContext.config` attribute.
+
+    .. versionadded:: 0.6.0
+
+    .. seealso::
+
+        :meth:`.EnvironmentContext.get_x_argument`
+
+    """
 
     config_file_name = None
     """Filesystem path to the .ini file in use."""
@@ -73,7 +89,7 @@ class Config(object):
     def print_stdout(self, text, *arg):
         """Render a message to standard out."""
 
-        self.stdout.write((str(text) % arg) + "\n")
+        self.stdout.write((compat.text_type(text) % arg) + "\n")
 
     @util.memoized_property
     def file_config(self):
@@ -90,7 +106,7 @@ class Config(object):
             here = os.path.abspath(os.path.dirname(self.config_file_name))
         else:
             here = ""
-        file_config = ConfigParser.SafeConfigParser({'here': here})
+        file_config = SafeConfigParser({'here': here})
         if self.config_file_name:
             file_config.read([self.config_file_name])
         else:
@@ -192,6 +208,19 @@ class CommandLine(object):
                                 help="Populate revision script with candidate "
                                     "migration operations, based on comparison "
                                     "of database to model.")
+            # "current" command
+            if 'head_only' in kwargs:
+                parser.add_argument("--head-only",
+                                    action="store_true",
+                                    help="Only show current version and "
+                                    "whether or not this is the head revision.")
+
+            if 'rev_range' in kwargs:
+                parser.add_argument("-r", "--rev-range",
+                                    action="store",
+                                    help="Specify a revision range; "
+                                    "format is [start]:[end]")
+
 
             positional_help = {
                 'directory': "location of scripts directory",
@@ -210,6 +239,11 @@ class CommandLine(object):
                             default="alembic",
                             help="Name of section in .ini file to "
                                     "use for Alembic config")
+        parser.add_argument("-x", action="append",
+                            help="Additional arguments consumed by "
+                            "custom env.py scripts, e.g. -x "
+                            "setting1=somesetting -x setting2=somesetting")
+
         subparsers = parser.add_subparsers()
 
         for fn in [getattr(command, n) for n in dir(command)]:
@@ -240,17 +274,24 @@ class CommandLine(object):
                         *[getattr(options, k) for k in positional],
                         **dict((k, getattr(options, k)) for k in kwarg)
                     )
-        except util.CommandError, e:
+        except util.CommandError as e:
             util.err(str(e))
 
     def main(self, argv=None):
         options = self.parser.parse_args(argv)
-
-        cfg = Config(options.config, options.name)
-        self.run_cmd(cfg, options)
+        if not hasattr(options, "cmd"):
+            # see http://bugs.python.org/issue9253, argparse
+            # behavior changed incompatibly in py3.3
+            self.parser.error("too few arguments")
+        else:
+            cfg = Config(file_=options.config,
+                            ini_section=options.name, cmd_opts=options)
+            self.run_cmd(cfg, options)
 
 def main(argv=None, prog=None, **kwargs):
     """The console runner function for Alembic."""
 
     CommandLine(prog=prog).main(argv=argv)
 
+if __name__ == '__main__':
+    main()
\ No newline at end of file
