@@ -1,9 +1,10 @@
-from alembic.ddl.impl import DefaultImpl
-from alembic.ddl.base import alter_table, AddColumn, ColumnName, \
+from sqlalchemy.ext.compiler import compiles
+
+from .. import util
+from .impl import DefaultImpl
+from .base import alter_table, AddColumn, ColumnName, \
     format_table_name, format_column_name, ColumnNullable, alter_column,\
     format_server_default,ColumnDefault, format_type, ColumnType
-from alembic import util
-from sqlalchemy.ext.compiler import compiles
 
 class MSSQLImpl(DefaultImpl):
     __dialect__ = 'mssql'
@@ -156,7 +157,7 @@ def visit_column_default(element, compiler, **kw):
 
 @compiles(ColumnName, 'mssql')
 def visit_rename_column(element, compiler, **kw):
-    return "EXEC sp_rename '%s.%s', '%s', 'COLUMN'" % (
+    return "EXEC sp_rename '%s.%s', %s, 'COLUMN'" % (
         format_table_name(compiler, element.table_name, element.schema),
         format_column_name(compiler, element.column_name),
         format_column_name(compiler, element.newname)
