@@ -1,9 +1,9 @@
-import alembic
-from alembic.operations import Operations
-from alembic.migration import MigrationContext
-from alembic import util
 from contextlib import contextmanager
 
+from .operations import Operations
+from .migration import MigrationContext
+from . import util
+
 class EnvironmentContext(object):
     """Represent the state made available to an ``env.py`` script.
 
@@ -99,12 +99,14 @@ class EnvironmentContext(object):
         be made available as ``from alembic import context``.
 
         """
-        alembic.context._install_proxy(self)
+        from .context import _install_proxy
+        _install_proxy(self)
         return self
 
     def __exit__(self, *arg, **kw):
-        alembic.context._remove_proxy()
-        alembic.op._remove_proxy()
+        from . import context, op
+        context._remove_proxy()
+        op._remove_proxy()
 
     def is_offline_mode(self):
         """Return True if the current migrations environment
@@ -195,9 +197,62 @@ class EnvironmentContext(object):
         This function does not require that the :class:`.MigrationContext`
         has been configured.
 
+        .. seealso::
+
+            :meth:`.EnvironmentContext.get_x_argument` - a newer and more
+            open ended system of extending ``env.py`` scripts via the command
+            line.
+
         """
         return self.context_opts.get('tag', None)
 
+    def get_x_argument(self, as_dictionary=False):
+        """Return the value(s) passed for the ``-x`` argument, if any.
+
+        The ``-x`` argument is an open ended flag that allows any user-defined
+        value or values to be passed on the command line, then available
+        here for consumption by a custom ``env.py`` script.
+
+        The return value is a list, returned directly from the ``argparse``
+        structure.  If ``as_dictionary=True`` is passed, the ``x`` arguments
+        are parsed using ``key=value`` format into a dictionary that is
+        then returned.
+
+        For example, to support passing a database URL on the command line,
+        the standard ``env.py`` script can be modified like this::
+
+            cmd_line_url = context.get_x_argument(as_dictionary=True).get('dbname')
+            if cmd_line_url:
+                engine = create_engine(cmd_line_url)
+            else:
+                engine = engine_from_config(
+                        config.get_section(config.config_ini_section),
+                        prefix='sqlalchemy.',
+                        poolclass=pool.NullPool)
+
+        This then takes effect by running the ``alembic`` script as::
+
+            alembic -x dbname=postgresql://user:pass@host/dbname upgrade head
+
+        This function does not require that the :class:`.MigrationContext`
+        has been configured.
+
+        .. versionadded:: 0.6.0
+
+        .. seealso::
+
+            :meth:`.EnvironmentContext.get_tag_argument`
+
+            :attr:`.Config.cmd_opts`
+
+        """
+        value = self.config.cmd_opts.x or []
+        if as_dictionary:
+            value = dict(
+                        arg.split('=', 1) for arg in value
+                    )
+        return value
+
     def configure(self,
             connection=None,
             url=None,
@@ -209,9 +264,11 @@ class EnvironmentContext(object):
             template_args=None,
             target_metadata=None,
             include_symbol=None,
+            include_object=None,
             include_schemas=False,
             compare_type=False,
             compare_server_default=False,
+            render_item=None,
             upgrade_token="upgrades",
             downgrade_token="downgrades",
             alembic_module_prefix="op.",
@@ -272,6 +329,11 @@ class EnvironmentContext(object):
          the :class:`.Config`
          object.  The value here overrides that of the :class:`.Config`
          object.
+        :param output_encoding: when using ``--sql`` to generate SQL
+         scripts, apply this encoding to the string output.
+
+         .. versionadded:: 0.5.0
+
         :param starting_rev: Override the "starting revision" argument
          when using ``--sql`` mode.
         :param tag: a string tag for usage by custom ``env.py`` scripts.
@@ -281,9 +343,16 @@ class EnvironmentContext(object):
          running the "revision" command.   Note that the script environment
          is only run within the "revision" command if the --autogenerate
          option is used, or if the option "revision_environment=true"
-         is present in the alembic.ini file.  New in 0.3.3.
+         is present in the alembic.ini file.
+
+         .. versionadded:: 0.3.3
+
         :param version_table: The name of the Alembic version table.
          The default is ``'alembic_version'``.
+        :param version_table_schema: Optional schema to place version
+         table within.
+
+         .. versionadded:: 0.5.0
 
         Parameters specific to the autogenerate feature, when
         ``alembic revision`` is run with the ``--autogenerate`` feature:
@@ -357,9 +426,59 @@ class EnvironmentContext(object):
          execute
          the two defaults on the database side to compare for equivalence.
 
+        :param include_object: A callable function which is given
+         the chance to return ``True`` or ``False`` for any object,
+         indicating if the given object should be considered in the
+         autogenerate sweep.
+
+         The function accepts the following positional arguments:
+
+         * ``object``: a :class:`~sqlalchemy.schema.SchemaItem` object such as a
+           :class:`~sqlalchemy.schema.Table` or :class:`~sqlalchemy.schema.Column`
+           object
+         * ``name``: the name of the object. This is typically available
+           via ``object.name``.
+         * ``type``: a string describing the type of object; currently
+           ``"table"`` or ``"column"``, but will include other types in a
+           future release
+         * ``reflected``: ``True`` if the given object was produced based on
+           table reflection, ``False`` if it's from a local :class:`.MetaData`
+           object.
+         * ``compare_to``: the object being compared against, if available,
+           else ``None``.
+
+         E.g.::
+
+            def include_object(object, name, type_, reflected, compare_to):
+                if (type_ == "column" and
+                    not reflected and
+                    object.info.get("skip_autogenerate", False)):
+                    return False
+                else:
+                    return True
+
+            context.configure(
+                # ...
+                include_object = include_object
+            )
+
+         The ``include_object`` filter will be expanded in a future release
+         to also receive type, constraint, and default objects.
+
+         .. versionadded:: 0.6.0
+
+         .. seealso::
+
+            ``include_schemas``, ``include_symbol``
+
+
         :param include_symbol: A callable function which, given a table name
          and schema name (may be ``None``), returns ``True`` or ``False``, indicating
          if the given table should be considered in the autogenerate sweep.
+
+         .. deprecated:: 0.6.0 ``include_symbol`` is superceded by the
+            more generic ``include_object`` parameter.
+
          E.g.::
 
             def include_symbol(tablename, schema):
@@ -387,6 +506,51 @@ class EnvironmentContext(object):
          .. versionchanged:: 0.4.0  the ``include_symbol`` callable must now
             also accept a "schema" argument, which may be None.
 
+         .. seealso::
+
+            ``include_schemas``, ``include_object``
+
+        :param include_schemas: If True, autogenerate will scan across
+         all schemas located by the SQLAlchemy
+         :meth:`~sqlalchemy.engine.reflection.Inspector.get_schema_names`
+         method, and include all differences in tables found across all
+         those schemas.  When using this option, you may want to also
+         use the ``include_symbol`` option to specify a callable which
+         can filter the tables/schemas that get included.
+
+         .. versionadded :: 0.4.0
+
+         .. seealso::
+
+            ``include_symbol``, ``include_object``
+
+        :param render_item: Callable that can be used to override how
+         any schema item, i.e. column, constraint, type,
+         etc., is rendered for autogenerate.  The callable receives a
+         string describing the type of object, the object, and
+         the autogen context.  If it returns False, the
+         default rendering method will be used.  If it returns None,
+         the item will not be rendered in the context of a Table
+         construct, that is, can be used to skip columns or constraints
+         within op.create_table()::
+
+            def my_render_column(type_, col, autogen_context):
+                if type_ == "column" and isinstance(col, MySpecialCol):
+                    return repr(col)
+                else:
+                    return False
+
+            context.configure(
+                # ...
+                render_item = my_render_column
+            )
+
+         Available values for the type string include: ``column``,
+         ``primary_key``, ``foreign_key``, ``unique``, ``check``,
+         ``type``, ``server_default``.
+
+         .. versionadded:: 0.5.0
+
         :param upgrade_token: When autogenerate completes, the text of the
          candidate upgrade operations will be present in this template
          variable when ``script.py.mako`` is rendered.  Defaults to
@@ -411,15 +575,6 @@ class EnvironmentContext(object):
          will render them using the dialect module name, i.e. ``mssql.BIT()``,
          ``postgresql.UUID()``.
 
-        :param include_schemas: If True, autogenerate will scan across
-         all schemas located by the SQLAlchemy
-         :meth:`~sqlalchemy.engine.reflection.Inspector.get_schema_names`
-         method, and include all differences in tables found across all
-         those schemas.  When using this option, you may want to also
-         use the ``include_symbol`` option to specify a callable which
-         can filter the tables/schemas that get included.
-
-         .. versionadded :: 0.4.0
 
         Parameters specific to individual backends:
 
@@ -451,11 +606,14 @@ class EnvironmentContext(object):
             opts['template_args'].update(template_args)
         opts['target_metadata'] = target_metadata
         opts['include_symbol'] = include_symbol
+        opts['include_object'] = include_object
         opts['include_schemas'] = include_schemas
         opts['upgrade_token'] = upgrade_token
         opts['downgrade_token'] = downgrade_token
         opts['sqlalchemy_module_prefix'] = sqlalchemy_module_prefix
         opts['alembic_module_prefix'] = alembic_module_prefix
+        if render_item is not None:
+            opts['render_item'] = render_item
         if compare_type is not None:
             opts['compare_type'] = compare_type
         if compare_server_default is not None:
