@@ -1,12 +1,14 @@
-from alembic import util
-from sqlalchemy import MetaData, Table, Column, String, literal_column, \
-    text
-from sqlalchemy import create_engine
-from alembic import ddl
+import io
+import logging
 import sys
+
+from sqlalchemy import MetaData, Table, Column, String, literal_column
+from sqlalchemy import create_engine
 from sqlalchemy.engine import url as sqla_url
 
-import logging
+from .compat import callable
+from . import ddl, util
+
 log = logging.getLogger(__name__)
 
 class MigrationContext(object):
@@ -59,8 +61,8 @@ class MigrationContext(object):
         self.dialect = dialect
         self.script = opts.get('script')
 
-        as_sql=opts.get('as_sql', False)
-        transactional_ddl=opts.get("transactional_ddl")
+        as_sql = opts.get('as_sql', False)
+        transactional_ddl = opts.get("transactional_ddl")
 
         if as_sql:
             self.connection = self._stdout_connection(connection)
@@ -71,15 +73,22 @@ class MigrationContext(object):
         self.as_sql = as_sql
         self.output_buffer = opts.get("output_buffer", sys.stdout)
 
+        if opts.get('output_encoding'):
+            self.output_buffer = io.TextIOWrapper(
+                                    self.output_buffer,
+                                    opts['output_encoding']
+                                )
+
         self._user_compare_type = opts.get('compare_type', False)
         self._user_compare_server_default = opts.get(
                                             'compare_server_default',
                                             False)
-
         version_table = opts.get('version_table', 'alembic_version')
+        version_table_schema = opts.get('version_table_schema', None)
         self._version = Table(
             version_table, MetaData(),
-            Column('version_num', String(32), nullable=False))
+            Column('version_num', String(32), nullable=False),
+            schema=version_table_schema)
 
         self._start_from_rev = opts.get("starting_rev")
         self.impl = ddl.DefaultImpl.get_by_dialect(dialect)(
@@ -195,14 +204,18 @@ class MigrationContext(object):
         """
         current_rev = rev = False
         self.impl.start_migrations()
-        for change, prev_rev, rev in self._migrations_fn(
-                                        self.get_current_revision(),
-                                        self):
+        for change, prev_rev, rev, doc in self._migrations_fn(
+                                            self.get_current_revision(),
+                                            self):
             if current_rev is False:
                 current_rev = prev_rev
                 if self.as_sql and not current_rev:
                     self._version.create(self.connection)
-            log.info("Running %s %s -> %s", change.__name__, prev_rev, rev)
+            if doc:
+                log.info("Running %s %s -> %s, %s", change.__name__, prev_rev,
+                    rev, doc)
+            else:
+                log.info("Running %s %s -> %s", change.__name__, prev_rev, rev)
             if self.as_sql:
                 self.impl.static_output(
                         "-- Running %s %s -> %s" %
@@ -279,7 +292,8 @@ class MigrationContext(object):
 
     def _compare_server_default(self, inspector_column,
                             metadata_column,
-                            rendered_metadata_default):
+                            rendered_metadata_default,
+                            rendered_column_default):
 
         if self._user_compare_server_default is False:
             return False
@@ -289,7 +303,7 @@ class MigrationContext(object):
                     self,
                     inspector_column,
                     metadata_column,
-                    inspector_column['default'],
+                    rendered_column_default,
                     metadata_column.server_default,
                     rendered_metadata_default
             )
@@ -299,5 +313,6 @@ class MigrationContext(object):
         return self.impl.compare_server_default(
                                 inspector_column,
                                 metadata_column,
-                                rendered_metadata_default)
+                                rendered_metadata_default,
+                                rendered_column_default)
 
