@@ -1,31 +1,30 @@
-from __future__ import with_statement
 
-from sqlalchemy.engine import url, default
-import shutil
+import io
 import os
-import itertools
-from sqlalchemy import create_engine, text, MetaData
-from alembic import util
-from alembic.migration import MigrationContext
-from alembic.environment import EnvironmentContext
 import re
-import alembic
-from alembic.operations import Operations
-from alembic.script import ScriptDirectory, Script
-from alembic import ddl
-import StringIO
-from alembic.ddl.impl import _impls
-import ConfigParser
-from nose import SkipTest
-from sqlalchemy.exc import SQLAlchemyError
-from sqlalchemy.util import decorator
 import shutil
 import textwrap
+from alembic.compat import configparser
+
+from nose import SkipTest
+from sqlalchemy.engine import default
+from sqlalchemy import create_engine, text
+from sqlalchemy.exc import SQLAlchemyError
+from sqlalchemy.util import decorator
+
+import alembic
+from alembic import util
+from alembic.compat import string_types, text_type
+from alembic.migration import MigrationContext
+from alembic.environment import EnvironmentContext
+from alembic.operations import Operations
+from alembic.script import ScriptDirectory, Script
+from alembic.ddl.impl import _impls
 
 staging_directory = os.path.join(os.path.dirname(__file__), 'scratch')
 files_directory = os.path.join(os.path.dirname(__file__), 'files')
 
-testing_config = ConfigParser.ConfigParser()
+testing_config = configparser.ConfigParser()
 testing_config.read(['test.cfg'])
 
 def sqlite_db():
@@ -49,15 +48,15 @@ def db_for_dialect(name):
     else:
         try:
             cfg = testing_config.get("db", name)
-        except ConfigParser.NoOptionError:
+        except configparser.NoOptionError:
             raise SkipTest("No dialect %r in test.cfg" % name)
         try:
-            eng = create_engine(cfg) #, echo=True)
-        except ImportError, er1:
+            eng = create_engine(cfg)
+        except ImportError as er1:
             raise SkipTest("Can't import DBAPI: %s" % er1)
         try:
-            conn = eng.connect()
-        except SQLAlchemyError, er2:
+            eng.connect()
+        except SQLAlchemyError as er2:
             raise SkipTest("Can't connect to database: %s" % er2)
         _engs[name] = eng
         return eng
@@ -68,6 +67,12 @@ def requires_07(fn, *arg, **kw):
         raise SkipTest("SQLAlchemy 0.7 required")
     return fn(*arg, **kw)
 
+@decorator
+def requires_08(fn, *arg, **kw):
+    if not util.sqla_08:
+        raise SkipTest("SQLAlchemy 0.8.0b2 or greater required")
+    return fn(*arg, **kw)
+
 _dialects = {}
 def _get_dialect(name):
     if name is None or name == 'default':
@@ -85,25 +90,27 @@ def _get_dialect(name):
 def assert_compiled(element, assert_string, dialect=None):
     dialect = _get_dialect(dialect)
     eq_(
-        unicode(element.compile(dialect=dialect)).\
+        text_type(element.compile(dialect=dialect)).\
                     replace("\n", "").replace("\t", ""),
         assert_string.replace("\n", "").replace("\t", "")
     )
 
 def capture_context_buffer(**kw):
-    buf = StringIO.StringIO()
+    if kw.pop('bytes_io', False):
+        buf = io.BytesIO()
+    else:
+        buf = io.StringIO()
 
     class capture(object):
         def __enter__(self):
             EnvironmentContext._default_opts = {
-                'dialect_name':"sqlite",
-                'output_buffer':buf
+                'dialect_name': "sqlite",
+                'output_buffer': buf
             }
             EnvironmentContext._default_opts.update(kw)
             return buf
 
         def __exit__(self, *arg, **kwarg):
-            print buf.getvalue()
             EnvironmentContext._default_opts = None
 
     return capture()
@@ -131,9 +138,9 @@ def assert_raises_message(except_cls, msg, callable_, *args, **kwargs):
     try:
         callable_(*args, **kwargs)
         assert False, "Callable did not raise an exception"
-    except except_cls, e:
+    except except_cls as e:
         assert re.search(msg, str(e)), "%r !~ %s" % (msg, e)
-        print str(e)
+        print(text_type(e))
 
 def op_fixture(dialect='default', as_sql=False):
     impl = _impls[dialect]
@@ -147,9 +154,9 @@ def op_fixture(dialect='default', as_sql=False):
             # as tests get more involved
             self.connection = None
         def _exec(self, construct, *args, **kw):
-            if isinstance(construct, basestring):
+            if isinstance(construct, string_types):
                 construct = text(construct)
-            sql = unicode(construct.compile(dialect=self.dialect))
+            sql = text_type(construct.compile(dialect=self.dialect))
             sql = re.sub(r'[\n\t]', '', sql)
             self.assertion.append(
                 sql
@@ -299,11 +306,15 @@ def clear_staging_env():
     shutil.rmtree(staging_directory, True)
 
 
-def write_script(scriptdir, rev_id, content):
+def write_script(scriptdir, rev_id, content, encoding='ascii'):
     old = scriptdir._revision_map[rev_id]
     path = old.path
-    with open(path, 'w') as fp:
-        fp.write(textwrap.dedent(content))
+
+    content = textwrap.dedent(content)
+    if encoding:
+        content = content.encode(encoding)
+    with open(path, 'wb') as fp:
+        fp.write(content)
     pyc_path = util.pyc_file_from_path(path)
     if os.access(pyc_path, os.F_OK):
         os.unlink(pyc_path)
@@ -323,7 +334,8 @@ def three_rev_fixture(cfg):
 
     script = ScriptDirectory.from_config(cfg)
     script.generate_revision(a, "revision a", refresh=True)
-    write_script(script, a, """
+    write_script(script, a, """\
+"Rev A"
 revision = '%s'
 down_revision = None
 
@@ -338,7 +350,8 @@ def downgrade():
 """ % a)
 
     script.generate_revision(b, "revision b", refresh=True)
-    write_script(script, b, """
+    write_script(script, b, """\
+"Rev B"
 revision = '%s'
 down_revision = '%s'
 
@@ -353,7 +366,8 @@ def downgrade():
 """ % (b, a))
 
     script.generate_revision(c, "revision c", refresh=True)
-    write_script(script, c, """
+    write_script(script, c, """\
+"Rev C"
 revision = '%s'
 down_revision = '%s'
 
@@ -366,4 +380,4 @@ def downgrade():
     op.execute("DROP STEP 3")
 
 """ % (c, b))
-    return a, b, c
\ No newline at end of file
+    return a, b, c
