@@ -1,18 +1,22 @@
+import re
+import sys
+from unittest import TestCase
+
 from sqlalchemy import MetaData, Column, Table, Integer, String, Text, \
     Numeric, CHAR, ForeignKey, DATETIME, \
     TypeDecorator, CheckConstraint, Unicode, Enum,\
-    UniqueConstraint, Boolean, ForeignKeyConstraint
+    UniqueConstraint, Boolean, ForeignKeyConstraint,\
+    PrimaryKeyConstraint
 from sqlalchemy.types import NULLTYPE, TIMESTAMP
 from sqlalchemy.dialects import mysql
 from sqlalchemy.engine.reflection import Inspector
 from sqlalchemy.sql import and_, column, literal_column
-from alembic import autogenerate
+
+from alembic import autogenerate, util
 from alembic.migration import MigrationContext
-from unittest import TestCase
-from tests import staging_env, sqlite_db, clear_staging_env, eq_, \
+from . import staging_env, sqlite_db, clear_staging_env, eq_, \
         eq_ignore_whitespace, requires_07, db_for_dialect
-import re
-import sys
+
 py3k = sys.version_info >= (3, )
 
 def _model_one(schema=None):
@@ -90,9 +94,17 @@ def _model_four():
 
     return m
 
-_default_include_symbol = lambda name, schema=None: name in ("parent", "child",
+def _default_include_object(obj, name, type_, reflected, compare_to):
+    if type_ == "table":
+        return name in ("parent", "child",
                                 "user", "order", "item",
                                 "address", "extra")
+    else:
+        return True
+
+_default_object_filters = [
+    _default_include_object
+]
 
 class AutogenTest(object):
     @classmethod
@@ -149,6 +161,8 @@ class ImplicitConstraintNoGenTest(AutogenTest, TestCase):
         Table('someothertable', m,
             Column('id', Integer, primary_key=True),
             Column('value', Boolean()),
+            mysql_engine='InnoDB',
+            mysql_default_charset='utf8'
         )
         return m
 
@@ -195,7 +209,10 @@ class ImplicitConstraintNoGenTest(AutogenTest, TestCase):
                 "nullable=False),\n"
             "    sa.Column('value', mysql.TINYINT(display_width=1), "
             "autoincrement=False, nullable=True),\n"
-            "    sa.PrimaryKeyConstraint('id')\n    )\n"
+            "    sa.PrimaryKeyConstraint('id'),\n"
+            "    mysql_default_charset='utf8',\n"
+            "    mysql_engine='InnoDB'\n"
+            "    )\n"
             "    op.drop_table('sometable')\n"
             "    ### end Alembic commands ###"
         )
@@ -235,9 +252,14 @@ class AutogenCrossSchemaTest(AutogenTest, TestCase):
         metadata = self.m2
         connection = self.context.bind
         diffs = []
+        def include_object(obj, name, type_, reflected, compare_to):
+            if type_ == "table":
+                return name == "t3"
+            else:
+                return True
         autogenerate._produce_net_changes(connection, metadata, diffs,
                                           self.autogen_context,
-                                          include_symbol=lambda n, s: n == 't3',
+                                          object_filters=[include_object],
                                           include_schemas=True
                                           )
         eq_(diffs[0][0], "add_table")
@@ -247,9 +269,14 @@ class AutogenCrossSchemaTest(AutogenTest, TestCase):
         metadata = self.m2
         connection = self.context.bind
         diffs = []
+        def include_object(obj, name, type_, reflected, compare_to):
+            if type_ == "table":
+                return name == "t4"
+            else:
+                return True
         autogenerate._produce_net_changes(connection, metadata, diffs,
                                           self.autogen_context,
-                                          include_symbol=lambda n, s: n == 't4',
+                                          object_filters=[include_object],
                                           include_schemas=True
                                           )
         eq_(diffs[0][0], "add_table")
@@ -259,9 +286,14 @@ class AutogenCrossSchemaTest(AutogenTest, TestCase):
         metadata = self.m2
         connection = self.context.bind
         diffs = []
+        def include_object(obj, name, type_, reflected, compare_to):
+            if type_ == "table":
+                return name == "t1"
+            else:
+                return True
         autogenerate._produce_net_changes(connection, metadata, diffs,
                                           self.autogen_context,
-                                          include_symbol=lambda n, s: n == 't1',
+                                          object_filters=[include_object],
                                           include_schemas=True
                                           )
         eq_(diffs[0][0], "remove_table")
@@ -271,9 +303,14 @@ class AutogenCrossSchemaTest(AutogenTest, TestCase):
         metadata = self.m2
         connection = self.context.bind
         diffs = []
+        def include_object(obj, name, type_, reflected, compare_to):
+            if type_ == "table":
+                return name == "t2"
+            else:
+                return True
         autogenerate._produce_net_changes(connection, metadata, diffs,
                                           self.autogen_context,
-                                          include_symbol=lambda n, s: n == 't2',
+                                          object_filters=[include_object],
                                           include_schemas=True
                                           )
         eq_(diffs[0][0], "remove_table")
@@ -306,7 +343,7 @@ class AutogenerateDiffTestWSchema(AutogenTest, TestCase):
         diffs = []
         autogenerate._produce_net_changes(connection, metadata, diffs,
                                           self.autogen_context,
-                                          include_symbol=_default_include_symbol,
+                                          object_filters=_default_object_filters,
                                           include_schemas=True
                                           )
 
@@ -381,7 +418,7 @@ class AutogenerateDiffTestWSchema(AutogenTest, TestCase):
         template_args = {}
         autogenerate._produce_migration_diffs(
                         self.context, template_args, set(),
-                        include_symbol=_default_include_symbol,
+                        include_object=_default_include_object,
                         include_schemas=True
                         )
         eq_(re.sub(r"u'", "'", template_args['upgrades']),
@@ -463,7 +500,7 @@ class AutogenerateDiffTest(AutogenTest, TestCase):
         diffs = []
         autogenerate._produce_net_changes(connection, metadata, diffs,
                                           self.autogen_context,
-                                          include_symbol= _default_include_symbol
+                                          object_filters=_default_object_filters,
                                     )
 
         eq_(
@@ -603,16 +640,63 @@ class AutogenerateDiffTest(AutogenTest, TestCase):
         )
         template_args = {}
         autogenerate._produce_migration_diffs(context, template_args, set())
+        template_args['upgrades'] = template_args['upgrades'].replace("u'", "'")
+        template_args['downgrades'] = template_args['downgrades'].\
+                                        replace("u'", "'")
         assert "alter_column('user'" not in template_args['upgrades']
         assert "alter_column('user'" not in template_args['downgrades']
         assert "alter_column('order'" in template_args['upgrades']
         assert "alter_column('order'" in template_args['downgrades']
 
+    def test_include_object(self):
+        def include_object(obj, name, type_, reflected, compare_to):
+            assert obj.name == name
+            if type_ == "table":
+                if reflected:
+                    assert obj.metadata is not self.m2
+                else:
+                    assert obj.metadata is self.m2
+                return name in ("address", "order")
+            elif type_ == "column":
+                if reflected:
+                    assert obj.table.metadata is not self.m2
+                else:
+                    assert obj.table.metadata is self.m2
+                return name != "street"
+            else:
+                return True
+
+
+        context = MigrationContext.configure(
+            connection=self.bind.connect(),
+            opts={
+                'compare_type': True,
+                'compare_server_default': True,
+                'target_metadata': self.m2,
+                'include_object': include_object,
+                'upgrade_token': "upgrades",
+                'downgrade_token': "downgrades",
+                'alembic_module_prefix': 'op.',
+                'sqlalchemy_module_prefix': 'sa.',
+            }
+        )
+        template_args = {}
+        autogenerate._produce_migration_diffs(context, template_args, set())
+        template_args['upgrades'] = template_args['upgrades'].replace("u'", "'")
+        template_args['downgrades'] = template_args['downgrades'].\
+                                        replace("u'", "'")
+
+        assert "alter_column('user'" not in template_args['upgrades']
+        assert "alter_column('user'" not in template_args['downgrades']
+        assert "'street'" not in template_args['upgrades']
+        assert "'street'" not in template_args['downgrades']
+        assert "alter_column('order'" in template_args['upgrades']
+        assert "alter_column('order'" in template_args['downgrades']
+
     def test_skip_null_type_comparison_reflected(self):
         diff = []
         autogenerate._compare_type(None, "sometable", "somecol",
-            {"name":"somecol", "type":NULLTYPE,
-            "nullable":True, "default":None},
+            Column("somecol", NULLTYPE),
             Column("somecol", Integer()),
             diff, self.autogen_context
         )
@@ -621,8 +705,7 @@ class AutogenerateDiffTest(AutogenTest, TestCase):
     def test_skip_null_type_comparison_local(self):
         diff = []
         autogenerate._compare_type(None, "sometable", "somecol",
-            {"name":"somecol", "type":Integer(),
-            "nullable":True, "default":None},
+            Column("somecol", Integer()),
             Column("somecol", NULLTYPE),
             diff, self.autogen_context
         )
@@ -640,8 +723,7 @@ class AutogenerateDiffTest(AutogenTest, TestCase):
 
         diff = []
         autogenerate._compare_type(None, "sometable", "somecol",
-            {"name":"somecol", "type":Integer(),
-            "nullable":True, "default":None},
+            Column("somecol", Integer, nullable=True),
             Column("somecol", MyType()),
             diff, self.autogen_context
         )
@@ -652,14 +734,57 @@ class AutogenerateDiffTest(AutogenTest, TestCase):
         from sqlalchemy.util import OrderedSet
         inspector = Inspector.from_engine(self.bind)
         autogenerate._compare_tables(
-            OrderedSet([(None, 'extra'), (None, 'user')]), OrderedSet(), inspector,
+            OrderedSet([(None, 'extra'), (None, 'user')]),
+            OrderedSet(), [], inspector,
                 MetaData(), diffs, self.autogen_context
         )
         eq_(
             [(rec[0], rec[1].name) for rec in diffs],
-            [('remove_table', 'extra'), ('remove_table', u'user')]
+            [('remove_table', 'extra'), ('remove_table', 'user')]
         )
 
+class AutogenKeyTest(AutogenTest, TestCase):
+    @classmethod
+    def _get_db_schema(cls):
+        m = MetaData()
+
+        Table('someothertable', m,
+            Column('id', Integer, primary_key=True),
+            Column('value', Integer, key="somekey"),
+        )
+        return m
+
+    @classmethod
+    def _get_model_schema(cls):
+        m = MetaData()
+
+        Table('sometable', m,
+            Column('id', Integer, primary_key=True),
+            Column('value', Integer, key="someotherkey"),
+        )
+        Table('someothertable', m,
+            Column('id', Integer, primary_key=True),
+            Column('value', Integer, key="somekey"),
+            Column("othervalue", Integer, key="otherkey")
+        )
+        return m
+
+    symbols = ['someothertable', 'sometable']
+    def test_autogen(self):
+        metadata = self.m2
+        connection = self.context.bind
+
+        diffs = []
+
+        autogenerate._produce_net_changes(connection, metadata, diffs,
+                                          self.autogen_context,
+                                          include_schemas=False
+                                          )
+        eq_(diffs[0][0], "add_table")
+        eq_(diffs[0][1].name, "sometable")
+        eq_(diffs[1][0], "add_column")
+        eq_(diffs[1][3].key, "otherkey")
+
 class AutogenerateDiffOrderTest(TestCase):
     @classmethod
     @requires_07
@@ -823,6 +948,23 @@ class AutogenRenderTest(TestCase):
             ")"
         )
 
+    def test_render_addtl_args(self):
+        m = MetaData()
+        t = Table('test', m,
+            Column('id', Integer, primary_key=True),
+            Column('q', Integer, ForeignKey('bar.address.id')),
+            postgresql_arg1="some_arg", mysql_engine="InnoDB"
+        )
+        eq_ignore_whitespace(
+            autogenerate._add_table(t, self.autogen_context),
+            "op.create_table('test',"
+            "sa.Column('id', sa.Integer(), nullable=False),"
+            "sa.Column('q', sa.Integer(), nullable=True),"
+            "sa.ForeignKeyConstraint(['q'], ['bar.address.id'], ),"
+            "sa.PrimaryKeyConstraint('id'),"
+            "mysql_engine='InnoDB',postgresql_arg1='some_arg')"
+        )
+
     def test_render_drop_table(self):
         eq_(
             autogenerate._drop_table(Table("sometable", MetaData()),
@@ -909,6 +1051,37 @@ class AutogenRenderTest(TestCase):
                 'nullable=False)'
         )
 
+    def test_render_custom(self):
+
+        def render(type_, obj, context):
+            if type_ == "foreign_key":
+                return None
+            if type_ == "column":
+                if obj.name == "y":
+                    return None
+                else:
+                    return "col(%s)" % obj.name
+            return "render:%s" % type_
+
+        autogen_context = {"opts": {
+            'render_item': render,
+            'alembic_module_prefix': 'sa.'
+        }}
+
+        t = Table('t', MetaData(),
+                Column('x', Integer),
+                Column('y', Integer),
+                PrimaryKeyConstraint('x'),
+                ForeignKeyConstraint(['x'], ['y'])
+            )
+        result = autogenerate._add_table(
+                    t, autogen_context
+                )
+        eq_(
+            result, """sa.create_table('t',
+col(x),
+render:primary_key\n)"""
+        )
 
     def test_render_modify_type(self):
         eq_ignore_whitespace(
@@ -960,36 +1133,59 @@ class AutogenRenderTest(TestCase):
         t2 = Table('t2', m, Column('c_rem', Integer))
 
         fk = ForeignKeyConstraint([t1.c.c], [t2.c.c_rem], onupdate="CASCADE")
+        if not util.sqla_08:
+            t1.append_constraint(fk)
+
         eq_ignore_whitespace(
             autogenerate._render_constraint(fk, self.autogen_context),
             "sa.ForeignKeyConstraint(['c'], ['t2.c_rem'], onupdate='CASCADE')"
         )
 
         fk = ForeignKeyConstraint([t1.c.c], [t2.c.c_rem], ondelete="CASCADE")
+        if not util.sqla_08:
+            t1.append_constraint(fk)
+
         eq_ignore_whitespace(
             autogenerate._render_constraint(fk, self.autogen_context),
             "sa.ForeignKeyConstraint(['c'], ['t2.c_rem'], ondelete='CASCADE')"
         )
 
         fk = ForeignKeyConstraint([t1.c.c], [t2.c.c_rem], deferrable=True)
+        if not util.sqla_08:
+            t1.append_constraint(fk)
         eq_ignore_whitespace(
             autogenerate._render_constraint(fk, self.autogen_context),
             "sa.ForeignKeyConstraint(['c'], ['t2.c_rem'], deferrable=True)"
         )
 
         fk = ForeignKeyConstraint([t1.c.c], [t2.c.c_rem], initially="XYZ")
+        if not util.sqla_08:
+            t1.append_constraint(fk)
         eq_ignore_whitespace(
             autogenerate._render_constraint(fk, self.autogen_context),
             "sa.ForeignKeyConstraint(['c'], ['t2.c_rem'], initially='XYZ')"
         )
 
+    def test_render_fk_constraint_use_alter(self):
+        m = MetaData()
+        Table('t', m, Column('c', Integer))
+        t2 = Table('t2', m, Column('c_rem', Integer,
+                                ForeignKey('t.c', name="fk1", use_alter=True)))
+        const = list(t2.foreign_keys)[0].constraint
+
+        eq_ignore_whitespace(
+            autogenerate._render_constraint(const, self.autogen_context),
+            "sa.ForeignKeyConstraint(['c_rem'], ['t.c'], "
+                    "name='fk1', use_alter=True)"
+        )
+
     def test_render_check_constraint_literal(self):
         eq_ignore_whitespace(
             autogenerate._render_check_constraint(
-                CheckConstraint("im a constraint"),
+                CheckConstraint("im a constraint", name='cc1'),
                 self.autogen_context
             ),
-            "sa.CheckConstraint('im a constraint')"
+            "sa.CheckConstraint('im a constraint', name='cc1')"
         )
 
     def test_render_check_constraint_sqlexpr(self):
