@@ -1,27 +1,26 @@
-from tests import op_fixture, assert_raises_message
+from sqlalchemy import Integer, func
+
 from alembic import op, util
-from sqlalchemy import Integer, Column, ForeignKey, \
-            UniqueConstraint, Table, MetaData, String,\
-            func
-from sqlalchemy.sql import table
+from . import op_fixture, assert_raises_message
 
 def test_rename_column():
     context = op_fixture('mysql')
-    op.alter_column('t1', 'c1', name="c2", existing_type=Integer)
+    op.alter_column('t1', 'c1', new_column_name="c2", existing_type=Integer)
     context.assert_(
         'ALTER TABLE t1 CHANGE c1 c2 INTEGER NULL'
     )
 
 def test_rename_column_serv_default():
     context = op_fixture('mysql')
-    op.alter_column('t1', 'c1', name="c2", existing_type=Integer, existing_server_default="q")
+    op.alter_column('t1', 'c1', new_column_name="c2", existing_type=Integer,
+                        existing_server_default="q")
     context.assert_(
         "ALTER TABLE t1 CHANGE c1 c2 INTEGER NULL DEFAULT 'q'"
     )
 
 def test_rename_column_serv_compiled_default():
     context = op_fixture('mysql')
-    op.alter_column('t1', 'c1', name="c2", existing_type=Integer,
+    op.alter_column('t1', 'c1', new_column_name="c2", existing_type=Integer,
             existing_server_default=func.utc_thing(func.current_timestamp()))
     # this is not a valid MySQL default but the point is to just
     # test SQL expression rendering
@@ -31,7 +30,7 @@ def test_rename_column_serv_compiled_default():
 
 def test_rename_column_autoincrement():
     context = op_fixture('mysql')
-    op.alter_column('t1', 'c1', name="c2", existing_type=Integer,
+    op.alter_column('t1', 'c1', new_column_name="c2", existing_type=Integer,
                                 existing_autoincrement=True)
     context.assert_(
         'ALTER TABLE t1 CHANGE c1 c2 INTEGER NULL AUTO_INCREMENT'
@@ -39,7 +38,7 @@ def test_rename_column_autoincrement():
 
 def test_col_add_autoincrement():
     context = op_fixture('mysql')
-    op.alter_column('t1', 'c1', name="c2", existing_type=Integer,
+    op.alter_column('t1', 'c1', new_column_name="c2", existing_type=Integer,
                                 autoincrement=True)
     context.assert_(
         'ALTER TABLE t1 CHANGE c1 c2 INTEGER NULL AUTO_INCREMENT'
@@ -47,7 +46,7 @@ def test_col_add_autoincrement():
 
 def test_col_remove_autoincrement():
     context = op_fixture('mysql')
-    op.alter_column('t1', 'c1', name="c2", existing_type=Integer,
+    op.alter_column('t1', 'c1', new_column_name="c2", existing_type=Integer,
                                 existing_autoincrement=True,
                                 autoincrement=False)
     context.assert_(
@@ -86,7 +85,7 @@ def test_drop_fk():
 
 def test_drop_constraint_primary():
     context = op_fixture('mysql')
-    op.drop_constraint('primary', 't1',type='primary')
+    op.drop_constraint('primary', 't1', type_='primary')
     context.assert_(
         "ALTER TABLE t1 DROP PRIMARY KEY "
     )
@@ -122,4 +121,4 @@ def test_drop_generic_constraint():
         "No generic 'DROP CONSTRAINT' in MySQL - please "
         "specify constraint type",
         op.drop_constraint, "f1", "t1"
-    )
\ No newline at end of file
+    )
