@@ -1,13 +1,14 @@
-from __future__ import with_statement
-from tests import op_fixture, db_for_dialect, eq_, staging_env, \
-            clear_staging_env, _no_sql_testing_config,\
-            capture_context_buffer, requires_07, write_script
 from unittest import TestCase
+
 from sqlalchemy import DateTime, MetaData, Table, Column, text, Integer, String
 from sqlalchemy.engine.reflection import Inspector
+
 from alembic import command, util
 from alembic.migration import MigrationContext
 from alembic.script import ScriptDirectory
+from . import db_for_dialect, eq_, staging_env, \
+            clear_staging_env, _no_sql_testing_config,\
+            capture_context_buffer, requires_07, write_script
 
 class PGOfflineEnumTest(TestCase):
     @requires_07
@@ -196,9 +197,10 @@ class PostgresqlDefaultCompareTest(TestCase):
         cols = insp.get_columns(t1.name)
         ctx = self.autogen_context['context']
         return ctx.impl.compare_server_default(
-            cols[0],
+            None,
             col,
-            rendered)
+            rendered,
+            cols[0]['default'])
 
     def test_compare_current_timestamp(self):
         self._compare_default_roundtrip(
@@ -241,4 +243,4 @@ class PostgresqlDefaultCompareTest(TestCase):
         )
         assert not self._compare_default(
             t1, t2, t2.c.id, ""
-        )
\ No newline at end of file
+        )
